@@ -109,6 +109,7 @@
 ##' @param seed Randomization seed (discrete case only). If \code{NULL} the RNG seed is untouched by this routine.
 ##' @param parallel Run in parallel using the \code{parallel} package?
 ##' @param trace Trace progress?
+##' @param reverse Do calculations in opposite order to improve stability ? (currently enabled by default for \code{oneStepGaussianOffMode} method only)
 ##' @param ... Control parameters for OSA method
 ##' @return \code{data.frame} with OSA \emph{standardized} residuals
 ##' in column \code{residual}. Depending on the method the output may
@@ -143,6 +144,7 @@ oneStepPredict <- function(obj,
                            seed = 123,
                            parallel = FALSE,
                            trace = TRUE,
+                           reverse = (method == "oneStepGaussianOffMode"),
                            ...
                            ){
     if (missing(observation.name))
@@ -205,9 +207,15 @@ oneStepPredict <- function(obj,
     ## Find randomeffects character
     args$random <- names.random
     args$regexp <- FALSE
-    ## Move data$name to parameter$name
-    args$parameters[observation.name] <- args$data[observation.name]
-    args$data[observation.name] <- NULL
+    ## Do we need to change, or take derivatives wrt., observations?
+    ## (search the code to see if a method uses "observation(k,y)" or
+    ## just "observation(k)").
+    obs.are.variables <- (method != "cdf")
+    ## Move data$name to parameter$name if necessary
+    if (obs.are.variables) {
+        args$parameters[observation.name] <- args$data[observation.name]
+        args$data[observation.name] <- NULL
+    }
     ## Make data.term.indicator in parameter list
     if(!is.null(data.term.indicator)){
         one <- rep(1, length(obs))
@@ -217,6 +225,15 @@ oneStepPredict <- function(obj,
         } else {
             args$parameters[[data.term.indicator]] <- cbind(one)
         }
+        ## Set attribute to tell the order of observations
+        ord <- rep(NA, length(obs))
+        ord[conditional] <- 0 ## First (out of bounds)
+        ord[subset] <- seq_along(subset)
+        ord[unconditional] <- length(obs) + 1 ## Never (out of bounds)
+        if (any(is.na(ord))) {
+            stop("Failed to determine the order of obervations")
+        }
+        attr(args$parameters[[data.term.indicator]], "ord") <- as.double(ord - 1)
     }
     ## Pretend these are *not observed*:
     if(length(unconditional)>0){
@@ -240,7 +257,8 @@ oneStepPredict <- function(obj,
         factor(fac)
     }
     map <- list()
-    map[[observation.name]] <- makeFac(obs)
+    if (obs.are.variables)
+        map[[observation.name]] <- makeFac(obs)
     if(!is.null(data.term.indicator)){
         map[[data.term.indicator]] <- makeFac(args$parameters[[data.term.indicator]])
     }
@@ -299,8 +317,10 @@ oneStepPredict <- function(obj,
     ##   * nlcdf.lower
     ##   * nlcdf.upper
     applyMethod <- function(oneStepMethod){
-        pred <- do.call("rbind", lapply(1:length(subset), oneStepMethod))
-        pred <- as.data.frame(pred)
+        ord <- seq_along(subset)
+        if (reverse) ord <- rev(ord)
+        pred <- do.call("rbind", lapply(ord, oneStepMethod))
+        pred <- as.data.frame(pred)[ord, ]
         pred$Fx <- 1 / ( 1 + exp(pred$nlcdf.lower - pred$nlcdf.upper) )
         pred$px <- 1 / ( exp(-pred$nlcdf.lower + pred$nll) +
                          exp(-pred$nlcdf.upper + pred$nll) )
@@ -336,8 +356,10 @@ oneStepPredict <- function(obj,
             H <- optimHess(opt$par, f, g)
             c(observation=obs[index], mean=opt$par, sd=sqrt(1/H))
         }
-        pred <- do.call("rbind", lapply(1:length(subset), oneStepGaussian))
-        pred <- as.data.frame(pred)
+        ord <- seq_along(subset)
+        if (reverse) ord <- rev(ord)
+        pred <- do.call("rbind", lapply(ord, oneStepGaussian))
+        pred <- as.data.frame(pred)[ord, ]
         pred$residual <- (pred$observation-pred$mean)/pred$sd
     }
 
@@ -346,7 +368,6 @@ oneStepPredict <- function(obj,
         p <- newobj$par
         newobj$fn(p) ## Test eval
         newobj$env$random.start <- expression({last.par[random]})
-        nll0 <- newobj$fn(observation(0))
         oneStepGaussian <- function(k){
             tracefun(k)
             index <- subset[k]
@@ -358,8 +379,10 @@ oneStepPredict <- function(obj,
             }
             c(observation=obs[index], nll = f(obs[index]), grad = g(obs[index]))
         }
-        pred <- do.call("rbind", lapply(1:length(subset), oneStepGaussian))
-        pred <- as.data.frame(pred)
+        ord <- seq_along(subset)
+        if (reverse) ord <- rev(ord)
+        pred <- do.call("rbind", lapply(ord, oneStepGaussian))
+        pred <- as.data.frame(pred)[ord, ]
         ################### Convert value and gradient to residual
         ## Need Lambert W function: x = W(x) * exp( W(x) ) , x > 0
         ## Vectorized in x and tested on extreme cases W(.Machine$double.xmin)
@@ -381,6 +404,7 @@ oneStepPredict <- function(obj,
             R <- sign(grad) * Rabs
             R
         }
+        nll0 <- newobj$fn(observation(0))
         R <- getResid( diff( c(nll0, pred$nll) ), pred$grad )
         M <- pred$observation - ifelse(pred$grad != 0, R * (R / pred$grad), 0)
         pred$mean <- M
