@@ -40,20 +40,29 @@ struct config_struct{
   struct {
     bool getListElement;
   } debug;
+  struct {
+    /** \brief Reduce memory of sparse hessian even if reducing speed ? */
+    bool sparse_hessian_compress;
+    /** \brief Use atomic sparse log determinant (faster but limited order) ? */
+    bool atomic_sparse_log_determinant;
+  } tmbad;
+  bool autopar; /**< \brief Enable automatic parallization (if OpenMP is enabled) ? */
+  int nthreads; /**< \brief Number of OpenMP threads to use (if OpenMP is enabled) */
 
   int cmd;
   SEXP envir; /* PROTECTed because function argument - see
                  'TMBconfig' */
-  void set(const char* name, bool &var, bool default_value) CSKIP(
+  template<class T>
+  void set(const char* name, T &var, T default_value)
   {
     // cmd=0: set defaults in this struct.
     // cmd=1: copy from this struct to R.
     // cmd=2: copy from R to this struct.
-    SEXP name_symbol = install(name);
+    SEXP name_symbol = Rf_install(name);
     if (cmd==0) var = default_value;
-    if (cmd==1) defineVar(name_symbol, asSEXP(var), envir);
-    if (cmd==2) var = INTEGER(findVar(name_symbol, envir))[0];
-  })
+    if (cmd==1) Rf_defineVar(name_symbol, asSEXP(var), envir);
+    if (cmd==2) var = INTEGER(Rf_findVar(name_symbol, envir))[0];
+  }
 #define SET(name,value)set(#name,name,value);
   void set() CSKIP(
   {
@@ -64,6 +73,10 @@ struct config_struct{
     SET(optimize.instantly,true);
     SET(optimize.parallel,false);
     SET(tape.parallel,true);
+    SET(tmbad.sparse_hessian_compress,false);
+    SET(tmbad.atomic_sparse_log_determinant,true);
+    SET(autopar,false);
+    SET(nthreads,1);
   })
 #undef SET
   config_struct() CSKIP(
